import sys
from typing import List, Iterable, Tuple, Callable, Union, Dict
from functools import singledispatch

from AnyQt.QtCore import Qt, pyqtSignal as Signal, QStringListModel, \
    QAbstractItemModel
from AnyQt.QtWidgets import QDialog, QVBoxLayout, QComboBox, QCheckBox, \
    QDialogButtonBox, QSpinBox, QWidget, QApplication, QFormLayout, QLineEdit

from orangewidget import gui
from orangewidget.utils.combobox import _ComboBoxListDelegate
from orangewidget.widget import OWBaseWidget

KeyType = Tuple[str, str, str]
ValueType = Union[str, int, bool]
ValueRangeType = Union[Iterable, None]
SettingsType = Dict[str, Tuple[ValueRangeType, ValueType]]


class SettingsDialog(QDialog):
    """ A dialog for settings manipulation.

    Attributes
    ----------
    master : Union[QWidget, None]
        Parent widget.

    settings : Dict[str, Dict[str, SettingsType]]
        Collection of box names, label texts, parameter names,
        initial control values and possible control values.

    """
    setting_changed = Signal(object, object)

    def __init__(self, master: Union[QWidget, None],
                 settings: Dict[str, Dict[str, SettingsType]]):
        super().__init__(master, windowTitle="Visual Settings")
        self.__controls: Dict[KeyType, Tuple[QWidget, ValueType]] = {}
        self.__changed_settings: Dict[KeyType, ValueType] = {}
        self.setting_changed.connect(self.__on_setting_changed)

        layout = QVBoxLayout()
        layout.setContentsMargins(0, 0, 0, 0)
        self.setLayout(layout)

        self.main_box = gui.vBox(self, box=None)  # type: QWidget

        buttons = QDialogButtonBox(
            orientation=Qt.Horizontal,
            standardButtons=QDialogButtonBox.Close | QDialogButtonBox.Reset,
        )
        closeButton = buttons.button(QDialogButtonBox.Close)
        closeButton.clicked.connect(self.close)

        resetButton = buttons.button(QDialogButtonBox.Reset)
        resetButton.clicked.connect(self.__reset)
        resetButton.setAutoDefault(False)

        layout.addWidget(buttons)

        self.__initialize(settings)

    @property
    def changed_settings(self) -> Dict[KeyType, ValueType]:
        """
        Keys (box, label, parameter) and values for changed settings.

        Returns
        -------
        settings : Dict[KeyType, ValueType]
        """
        return self.__changed_settings

    def __on_setting_changed(self, key: KeyType, value: ValueType):
        self.__changed_settings[key] = value

    def __reset(self):
        for key in self.__changed_settings:
            _set_control_value(*self.__controls[key])
        self.__changed_settings = {}

    def __initialize(self, settings: Dict[str, Dict[str, SettingsType]]):
        for box_name in settings:
            box = gui.vBox(self.main_box, box=box_name)
            form = QFormLayout()
            form.setFormAlignment(Qt.AlignLeft | Qt.AlignTop)
            form.setLabelAlignment(Qt.AlignLeft)
            box.layout().addLayout(form)
            for label, values in settings[box_name].items():
                self.__add_row(form, box_name, label, values)
        self.main_box.adjustSize()

    def __add_row(self, form: QFormLayout, box_name: str,
                  label: str, settings: SettingsType):
        box = gui.hBox(None, box=None)
        for parameter, (values, default_value) in settings.items():
            key = (box_name, label, parameter)
            control = _add_control(values or default_value, default_value, key,
                                   self.setting_changed)
            control.setToolTip(parameter)
            box.layout().addWidget(control)
            self.__controls[key] = (control, default_value)
        form.addRow(f"{label}:", box)

    def apply_settings(self, settings: Iterable[Tuple[KeyType, ValueType]]):
        """ Assign values to controls.

        Parameters
        ----------
        settings : Iterable[Tuple[KeyType, ValueType]
            Collection of box names, label texts, parameter names
            and control values.
        """
        for key, value in settings:
            _set_control_value(self.__controls[key][0], value)

    def show_dlg(self):
        """ Open the dialog. """
        self.show()
        self.raise_()
        self.activateWindow()


class VisualSettingsDialog(SettingsDialog):
    """ A dialog for visual settings manipulation, that can be used along
    OWBaseWidget.

    The OWBaseWidget should implement set_visual_settings.
    If the OWBaseWidget has visual_settings property as Setting({}),
    the saved settings are applied.

    Attributes
    ----------
    master : OWBaseWidget
        Parent widget.

    settings : Dict[str, Dict[str, SettingsType]]
        Collection of box names, label texts, parameter names,
        initial control values and possible control values.
    """

    def __init__(self, master: OWBaseWidget,
                 settings: Dict[str, Dict[str, SettingsType]]):
        super().__init__(master, settings)
        self.setting_changed.connect(master.set_visual_settings)
        if hasattr(master, "visual_settings"):
            self.apply_settings(master.visual_settings.items())
        master.openVisualSettingsClicked.connect(self.show_dlg)


@singledispatch
def _add_control(*_):
    raise NotImplementedError


@_add_control.register(list)
def _(values: List[str], value: str, key: KeyType, signal: Callable) \
        -> QComboBox:
    combo = QComboBox()
    combo.addItems(values)
    combo.setCurrentText(value)
    combo.currentTextChanged.connect(lambda text: signal.emit(key, text))
    return combo


class FontList(list):
    pass


@_add_control.register(FontList)
def _(values: FontList, value: str, key: KeyType, signal: Callable) \
        -> QComboBox:
    class FontModel(QStringListModel):
        def data(self, index, role=Qt.DisplayRole):
            if role == Qt.AccessibleDescriptionRole \
                    and super().data(index, Qt.DisplayRole) == "":
                return "separator"

            value = super().data(index, role)
            if role == Qt.DisplayRole and value.startswith("."):
                value = value[1:]
            return value

        def flags(self, index):
            if index.data(Qt.DisplayRole) == "separator":
                return Qt.NoItemFlags
            else:
                return super().flags(index)

    combo = QComboBox()
    model = FontModel(values)
    combo.setModel(model)
    combo.setCurrentIndex(values.index(value))
    combo.currentIndexChanged.connect(lambda i: signal.emit(key, values[i]))
    combo.setItemDelegate(_ComboBoxListDelegate())
    return combo


@_add_control.register(range)
def _(values: Iterable[int], value: int, key: KeyType, signal: Callable) \
        -> QSpinBox:
    spin = QSpinBox(minimum=values.start, maximum=values.stop,
                    singleStep=values.step, value=value)
    spin.valueChanged.connect(lambda val: signal.emit(key, val))
    return spin


@_add_control.register(bool)
def _(_: bool, value: bool, key: KeyType, signal: Callable) -> QCheckBox:
    check = QCheckBox(text=f"{key[-1]} ", checked=value)
    check.stateChanged.connect(lambda val: signal.emit(key, bool(val)))
    return check


@_add_control.register(str)
def _(_: str, value: str, key: KeyType, signal: Callable) -> QLineEdit:
    line_edit = QLineEdit(value)
    line_edit.textChanged.connect(lambda text: signal.emit(key, text))
    return line_edit


@singledispatch
def _set_control_value(*_):
    raise NotImplementedError


@_set_control_value.register(QComboBox)
def _(combo: QComboBox, value: str):
    model: QAbstractItemModel = combo.model()
    values = [model.data(model.index(i, 0), role=Qt.EditRole)
              for i in range(model.rowCount())]
    combo.setCurrentIndex(values.index(value))


@_set_control_value.register(QSpinBox)
def _(spin: QSpinBox, value: int):
    spin.setValue(value)


@_set_control_value.register(QCheckBox)
def _(check: QCheckBox, value: bool):
    check.setChecked(value)


@_set_control_value.register(QLineEdit)
def _(edit: QLineEdit, value: str):
    edit.setText(value)


if __name__ == "__main__":
    from AnyQt.QtWidgets import QPushButton

    app = QApplication(sys.argv)
    w = QDialog()
    w.setFixedSize(400, 200)

    _items = ["Foo", "Bar", "Baz", "Foo Bar", "Foo Baz", "Bar Baz"]
    _settings = {
        "Box 1": {
            "Item 1": {
                "Parameter 1": (_items[:10], _items[0]),
                "Parameter 2": (_items[:10], _items[0]),
                "Parameter 3": (range(4, 20), 5)
            },
            "Item 2": {
                "Parameter 1": (_items[:10], _items[1]),
                "Parameter 2": (range(4, 20), 6),
                "Parameter 3": (range(4, 20), 7)
            },
            "Item 3": {
                "Parameter 1": (_items[:10], _items[1]),
                "Parameter 2": (range(4, 20), 8)
            },
        },
        "Box 2": {
            "Item 1": {
                "Parameter 1": (_items[:10], _items[0]),
                "Parameter 2": (None, True)
            },
            "Item 2": {
                "Parameter 1": (_items[:10], _items[1]),
                "Parameter 2": (None, False)
            },
            "Item 3": {
                "Parameter 1": (None, False),
                "Parameter 2": (None, True)
            },
            "Item 4": {
                "Parameter 1": (_items[:10], _items[0]),
                "Parameter 2": (None, False)
            },
            "Item 5": {
                "Parameter 1": ("", "Foo"),
                "Parameter 2": (None, False)
            },
            "Item 6": {
                "Parameter 1": ("", ""),
                "Parameter 2": (None, False)
            },
        },
    }

    dlg = SettingsDialog(w, _settings)
    dlg.setting_changed.connect(lambda *res: print(*res))
    dlg.finished.connect(lambda res: print(res, dlg.changed_settings))

    btn = QPushButton(w)
    btn.setText("Open dialog")
    btn.clicked.connect(dlg.show_dlg)

    w.show()
    sys.exit(app.exec())



import operator
from weakref import WeakKeyDictionary
from typing import TypeVar, Sequence, Optional, Callable, Any

T = TypeVar("T")


def index_of(
        seq: Sequence[T],
        el: T,
        eq: Callable[[T, T], bool] = operator.eq,
) -> Optional[int]:
    """
    Return index of `el` in `seq` where equality is defined by `eq`.

    Return `None` if not found.
    """
    for i, e in enumerate(seq):
        if eq(el, e):
            return i
    return None


class WeakKeyDefaultDict(WeakKeyDictionary):
    """
    A `WeakKeyDictionary` that also acts like a :class:`collections.defaultdict`
    """
    default_factory: Callable[[], Any]

    def __init__(self, default_factory: Callable[[], Any], *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.default_factory = default_factory

    def __getitem__(self, key):
        try:
            value = super().__getitem__(key)
        except KeyError:
            value = self.default_factory()
            self.__setitem__(key, value)
        return value



from AnyQt.QtGui import QFocusEvent
from AnyQt.QtWidgets import QStyle, QApplication
from orangewidget.tests.base import GuiTest
from orangewidget.utils import buttons


class SimpleButtonTest(GuiTest):
    def test_button(self):
        # Run through various state change and drawing code for coverage
        b = buttons.SimpleButton()
        b.setIcon(b.style().standardIcon(QStyle.SP_ComputerIcon))

        QApplication.sendEvent(b, QFocusEvent(QFocusEvent.FocusIn))
        QApplication.sendEvent(b, QFocusEvent(QFocusEvent.FocusOut))

        b.grab()
        b.setDown(True)
        b.grab()
        b.setCheckable(True)
        b.setChecked(True)
        b.grab()


class TestVariableTextPushButton(GuiTest):
    def test_button(self):
        b = buttons.VariableTextPushButton(
            textChoiceList=["", "A", "MMMMMMM"]
        )
        b.setText("")
        sh = b.sizeHint()
        b.setText("A")
        self.assertEqual(b.sizeHint(), sh)
        b.setText("MMMMMMM")
        self.assertEqual(b.sizeHint(), sh)
        b.setTextChoiceList(["A", "B", "C"])



import unittest

from AnyQt.QtCore import QStringListModel, Qt
from AnyQt.QtTest import QTest
from AnyQt.QtWidgets import QLineEdit

from orangewidget.tests.base import GuiTest
from orangewidget.utils.listview import ListViewSearch


class TestListViewSearch(GuiTest):
    def setUp(self) -> None:
        super().setUp()
        self.lv = ListViewSearch()
        s = ["one", "two", "three", "four"]
        model = QStringListModel(s)
        self.lv.setModel(model)

    def tearDown(self) -> None:
        super().tearDown()
        self.lv.deleteLater()
        self.lv = None

    def test_list_view(self):
        num_items = 4
        self.assertEqual(num_items, self.lv.model().rowCount())

        filter_row = self.lv.findChild(QLineEdit)
        filter_row.grab()
        self.lv.grab()

        QTest.keyClick(filter_row, Qt.Key_E, delay=-1)
        self.assertListEqual(
            [False, True, False, True],
            [self.lv.isRowHidden(i) for i in range(num_items)],
        )
        QTest.keyClick(filter_row, Qt.Key_Backspace)
        self.assertListEqual(
            [False] * 4, [self.lv.isRowHidden(i) for i in range(num_items)]
        )
        QTest.keyClick(filter_row, Qt.Key_F)
        self.assertListEqual(
            [True, True, True, False],
            [self.lv.isRowHidden(i) for i in range(num_items)],
        )
        QTest.keyClick(filter_row, Qt.Key_Backspace)
        QTest.keyClick(filter_row, Qt.Key_T)
        self.assertListEqual(
            [True, False, False, True],
            [self.lv.isRowHidden(i) for i in range(num_items)],
        )
        QTest.keyClick(filter_row, Qt.Key_H)
        self.assertListEqual(
            [True, True, False, True],
            [self.lv.isRowHidden(i) for i in range(num_items)],
        )

    def test_insert_new_value(self):
        num_items = 4
        filter_row = self.lv.findChild(QLineEdit)
        filter_row.grab()
        self.lv.grab()

        QTest.keyClick(filter_row, Qt.Key_E, delay=-1)
        self.assertListEqual(
            [False, True, False, True],
            [self.lv.isRowHidden(i) for i in range(num_items)],
        )

        model = self.lv.model()
        if model.insertRow(model.rowCount()):
            index = model.index(model.rowCount() - 1, 0)
            model.setData(index, "six")

        self.assertListEqual(
            [False, True, False, True, True],
            [self.lv.isRowHidden(i) for i in range(num_items + 1)],
        )

    def test_empty(self):
        self.lv.setModel(QStringListModel([]))
        self.assertEqual(0, self.lv.model().rowCount())

        filter_row = self.lv.findChild(QLineEdit)
        filter_row.grab()
        self.lv.grab()

        QTest.keyClick(filter_row, Qt.Key_T)
        QTest.keyClick(filter_row, Qt.Key_Backspace)


if __name__ == "__main__":
    unittest.main()



import pyqtgraph as pg

from orangewidget.tests.base import GuiTest
from orangewidget.utils.matplotlib_export import scatterplot_code


def add_intro(a):
    r = "import matplotlib.pyplot as plt\n" + \
        "from numpy import array\n" + \
        "plt.clf()"
    return r + a


class TestScatterPlot(GuiTest):
    def test_scatterplot_simple(self):
        plotWidget = pg.PlotWidget(background="w")
        scatterplot = pg.ScatterPlotItem()
        scatterplot.setData(x=[1, 2, 3], y=[3, 2, 1])
        plotWidget.addItem(scatterplot)
        code = scatterplot_code(scatterplot)
        self.assertIn("plt.scatter", code)
        exec(add_intro(code), {})



# THIS FILE IS GENERATED FROM ORANGE-WIDGET-BASE SETUP.PY
short_version = '4.17.0'
version = '4.17.0'
full_version = '4.17.0'
git_revision = 'd829100d1401fc116251f52b17c2590927beed3e'
release = True

if not release:
    version = full_version
    short_version += ".dev"
